import pdbfixer
import mdtraj as md
from simtk.openmm.app import PDBFile
from simtk.openmm.app.topology import Topology
import pickle

# Load the file into PDBFixer
fixer = pdbfixer.PDBFixer(filename='5udc_splitchain.pdb')

# Identify missing residues
fixer.findMissingResidues()

# Remove missing residues if they are part of terminal fragments 
chains = list(fixer.topology.chains())
keys = fixer.missingResidues.keys()
for key in list(keys):
    chain = chains[key[0]]
    if key[1] == 0 or key[1] == len(list(chain.residues())):
        if len(fixer.missingResidues[key]) > 10: # Do not add back terminal fragment if its longer than 10 residues
            if fixer.missingResidues[key][0] == 'LEU':
                fixer.missingResidues[key] = ['LEU'] # Add LEU back in, even though loop will not be added back in
            elif fixer.missingResidues[key][-1] == 'GLN':
                fixer.missingResidues[key] = ['GLN'] # Add GLN back in, even though loop will not be added back in
            else:
                print(key, chain, fixer.missingResidues[key])
                del fixer.missingResidues[key]

fixer.findNonstandardResidues()
fixer.nonstandardResidues
fixer.removeHeterogens(False)
fixer.findMissingAtoms()
fixer.addMissingAtoms()

PDBFile.writeFile(fixer.topology, fixer.positions, open('5udc_splitchain_clean.pdb', 'w'), keepIds=True)
